
/* File generated by CMake - Do not edit! */
char *locale_set[] = {
  "ar_AR",
  "bg_BG",
  "cs_CZ",
  "cy_CY",
  "da_DA",
  "de_DE",
  "en_GB",
  "en_US",
  "es_ES",
  "fa_IR",
  "fi_FI",
  "fr_FR",
  "he_IL",
  "hu_HU",
  "it_IT",
  "ja_JA",
  "ko_KR",
  "lt_LT",
  "lv_LV",
  "my_MY",
  "nb_NO",
  "nl_NL",
  "pl_PL",
  "pt_PT",
  "pt_BR",
  "ro_RO",
  "ru_RU",
  "sk_SK",
  "sl_SI",
  "sv_SE",
  "tr_TR",
  "vi_VN",
  "zh_CN",
  "zh_TW",
  (char*)0
};
