/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __IPC_GDBUS_H__
#define __IPC_GDBUS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.xiphos.remote */

#define XIPHOS_TYPE_REMOTE (xiphos_remote_get_type ())
#define XIPHOS_REMOTE(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), XIPHOS_TYPE_REMOTE, XiphosRemote))
#define XIPHOS_IS_REMOTE(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), XIPHOS_TYPE_REMOTE))
#define XIPHOS_REMOTE_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), XIPHOS_TYPE_REMOTE, XiphosRemoteIface))

struct _XiphosRemote;
typedef struct _XiphosRemote XiphosRemote;
typedef struct _XiphosRemoteIface XiphosRemoteIface;

struct _XiphosRemoteIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_get_current_reference) (
    XiphosRemote *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_next_search_reference) (
    XiphosRemote *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_set_current_reference) (
    XiphosRemote *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_reference);

  void (*navigation_signal) (
    XiphosRemote *object,
    const gchar *arg_reference);

  void (*search_performed_signal) (
    XiphosRemote *object,
    const gchar *arg_reference,
    gint arg_hits);

};

GType xiphos_remote_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *xiphos_remote_interface_info (void);
guint xiphos_remote_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void xiphos_remote_complete_set_current_reference (
    XiphosRemote *object,
    GDBusMethodInvocation *invocation);

void xiphos_remote_complete_get_current_reference (
    XiphosRemote *object,
    GDBusMethodInvocation *invocation,
    const gchar *reference);

void xiphos_remote_complete_get_next_search_reference (
    XiphosRemote *object,
    GDBusMethodInvocation *invocation,
    const gchar *reference);



/* D-Bus signal emissions functions: */
void xiphos_remote_emit_search_performed_signal (
    XiphosRemote *object,
    const gchar *arg_reference,
    gint arg_hits);

void xiphos_remote_emit_navigation_signal (
    XiphosRemote *object,
    const gchar *arg_reference);



/* D-Bus method calls: */
void xiphos_remote_call_set_current_reference (
    XiphosRemote *proxy,
    const gchar *arg_reference,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean xiphos_remote_call_set_current_reference_finish (
    XiphosRemote *proxy,
    GAsyncResult *res,
    GError **error);

gboolean xiphos_remote_call_set_current_reference_sync (
    XiphosRemote *proxy,
    const gchar *arg_reference,
    GCancellable *cancellable,
    GError **error);

void xiphos_remote_call_get_current_reference (
    XiphosRemote *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean xiphos_remote_call_get_current_reference_finish (
    XiphosRemote *proxy,
    gchar ** out_reference,
    GAsyncResult *res,
    GError **error);

gboolean xiphos_remote_call_get_current_reference_sync (
    XiphosRemote *proxy,
    gchar ** out_reference,
    GCancellable *cancellable,
    GError **error);

void xiphos_remote_call_get_next_search_reference (
    XiphosRemote *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean xiphos_remote_call_get_next_search_reference_finish (
    XiphosRemote *proxy,
    gchar ** out_reference,
    GAsyncResult *res,
    GError **error);

gboolean xiphos_remote_call_get_next_search_reference_sync (
    XiphosRemote *proxy,
    gchar ** out_reference,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define XIPHOS_TYPE_REMOTE_PROXY (xiphos_remote_proxy_get_type ())
#define XIPHOS_REMOTE_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), XIPHOS_TYPE_REMOTE_PROXY, XiphosRemoteProxy))
#define XIPHOS_REMOTE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), XIPHOS_TYPE_REMOTE_PROXY, XiphosRemoteProxyClass))
#define XIPHOS_REMOTE_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), XIPHOS_TYPE_REMOTE_PROXY, XiphosRemoteProxyClass))
#define XIPHOS_IS_REMOTE_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), XIPHOS_TYPE_REMOTE_PROXY))
#define XIPHOS_IS_REMOTE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), XIPHOS_TYPE_REMOTE_PROXY))

typedef struct _XiphosRemoteProxy XiphosRemoteProxy;
typedef struct _XiphosRemoteProxyClass XiphosRemoteProxyClass;
typedef struct _XiphosRemoteProxyPrivate XiphosRemoteProxyPrivate;

struct _XiphosRemoteProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  XiphosRemoteProxyPrivate *priv;
};

struct _XiphosRemoteProxyClass
{
  GDBusProxyClass parent_class;
};

GType xiphos_remote_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (XiphosRemoteProxy, g_object_unref)
#endif

void xiphos_remote_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
XiphosRemote *xiphos_remote_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
XiphosRemote *xiphos_remote_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void xiphos_remote_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
XiphosRemote *xiphos_remote_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
XiphosRemote *xiphos_remote_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define XIPHOS_TYPE_REMOTE_SKELETON (xiphos_remote_skeleton_get_type ())
#define XIPHOS_REMOTE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), XIPHOS_TYPE_REMOTE_SKELETON, XiphosRemoteSkeleton))
#define XIPHOS_REMOTE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), XIPHOS_TYPE_REMOTE_SKELETON, XiphosRemoteSkeletonClass))
#define XIPHOS_REMOTE_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), XIPHOS_TYPE_REMOTE_SKELETON, XiphosRemoteSkeletonClass))
#define XIPHOS_IS_REMOTE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), XIPHOS_TYPE_REMOTE_SKELETON))
#define XIPHOS_IS_REMOTE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), XIPHOS_TYPE_REMOTE_SKELETON))

typedef struct _XiphosRemoteSkeleton XiphosRemoteSkeleton;
typedef struct _XiphosRemoteSkeletonClass XiphosRemoteSkeletonClass;
typedef struct _XiphosRemoteSkeletonPrivate XiphosRemoteSkeletonPrivate;

struct _XiphosRemoteSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  XiphosRemoteSkeletonPrivate *priv;
};

struct _XiphosRemoteSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType xiphos_remote_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (XiphosRemoteSkeleton, g_object_unref)
#endif

XiphosRemote *xiphos_remote_skeleton_new (void);


G_END_DECLS

#endif /* __IPC_GDBUS_H__ */
