/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#ifndef __WK_MARSHAL_MARSHAL_H__
#define __WK_MARSHAL_MARSHAL_H__

#include <glib-object.h>

G_BEGIN_DECLS

/* VOID:POINTER,BOOLEAN (/build/xiphos/src/xiphos/src/webkit/marshal.list:1) */
extern
void wk_marshal_VOID__POINTER_BOOLEAN (GClosure     *closure,
                                       GValue       *return_value,
                                       guint         n_param_values,
                                       const GValue *param_values,
                                       gpointer      invocation_hint,
                                       gpointer      marshal_data);

/* VOID:STRING (/build/xiphos/src/xiphos/src/webkit/marshal.list:2) */
#define wk_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING

/* VOID:VOID (/build/xiphos/src/xiphos/src/webkit/marshal.list:3) */
#define wk_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID

/* VOID:STRING,INT (/build/xiphos/src/xiphos/src/webkit/marshal.list:4) */
extern
void wk_marshal_VOID__STRING_INT (GClosure     *closure,
                                  GValue       *return_value,
                                  guint         n_param_values,
                                  const GValue *param_values,
                                  gpointer      invocation_hint,
                                  gpointer      marshal_data);

/* VOID:POINTER (/build/xiphos/src/xiphos/src/webkit/marshal.list:5) */
#define wk_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER

/* BOOLEAN:POINTER,BOOLEAN (/build/xiphos/src/xiphos/src/webkit/marshal.list:6) */
extern
void wk_marshal_BOOLEAN__POINTER_BOOLEAN (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
                                          const GValue *param_values,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);


G_END_DECLS

#endif /* __WK_MARSHAL_MARSHAL_H__ */
